# -------------------------------------------------
#    Script for making vis-files from UVFITS/MS
#    Assumes CASA Version 4.1.0
# -------------------------------------------------
# [130902] - a.scaife@soton.ac.uk
# [151012] - luke.pratley.15@ucl.ac.uk
# -------------------------------------------------
# to run:
# my-laptop> casapy --nologger -c extract_vis_ATCA.py
# -------------------------------------------------


import numpy as np
import os,sys

const_c = 299792458. # m/s


# -----------------------------------------------------
# define input data files:
infile='0153-410'


# -----------------------------------------------------
# open MS SPECTRAL table:
tb.open(infile+'.ms/SPECTRAL_WINDOW',nomodify=True)
# extract channel frequencies (in Hz):
freq=tb.getcol("CHAN_FREQ")
refreq=tb.getcol("REF_FREQUENCY")
print ">>Extracting data for ",freq.shape[1]," frequencies: ",freq/1e9,"[GHz]"
tb.close


# -----------------------------------------------------
# open MS MAIN table:
tb.open(infile+'.ms',nomodify=True)
# extract UVW (note these are in metres):
uvw=tb.getcol("UVW")
units=tb.getcolkeyword("UVW", "QuantumUnits")
print ">>Units of UVW are: ",units[0]
print ">>There are ",uvw.shape[1]," visibilities per channel"
# extract complex data:
data=tb.getcol("DATA")
# extract weight for each visibility:
sigma=tb.getcol("SIGMA")

# -----------------------------------------------------
# open output vis-file:
os.system('rm -rf '+infile+'*.vis \n')
visfileI=open(infile+'I.vis','w')
visfileQ=open(infile+'Q.vis','w')
visfileU=open(infile+'U.vis','w')
visfileV=open(infile+'V.vis','w')
print 'Total Channels: ', len(data[0,:,0])
# Loop over frequency channels, and add uv data to .vis file
for j in range(0,len(data[0,:,0])):
	print 'Adding Channel:', j, 'Frequency (Hz):', freq[j, 0]
	flagXX = np.array(tb.getcol('FLAG')[0, j, :])
	flagXY = np.array(tb.getcol('FLAG')[1, j, :])
	flagYX = np.array(tb.getcol('FLAG')[2, j, :])
	flagYY = np.array(tb.getcol('FLAG')[3, j, :])
	# Assuming linear feed and correlation matrix of [[0 1], [2 3]]
	dataI = 0.5*(data[0,j,:] + data[3,j,:])
	dataU = 0.5*(data[1,j,:] + data[2,j,:])
	dataV = 0.5*(data[1,j,:] - data[2,j,:])/complex(0 , 1)
	dataQ = 0.5*(data[0,j,:] - data[3,j,:])
	sigmaI = np.sqrt((sigma[0,:]**2 + sigma[3,:]**2)/2.)
	sigmaU = np.sqrt((sigma[1,:]**2 + sigma[2,:]**2)/2.)
	sigmaV = np.sqrt((sigma[1,:]**2 + sigma[2,:]**2)/2.)
	sigmaQ = np.sqrt((sigma[0,:]**2 + sigma[3,:]**2)/2.)

	# -----------------------------------------------------
	print ">> Columns are:"
	print " U(lambda)  V(lambda)  Real(Jy)  Imag(Jy)  Sigma(Jy) "

	uvw_c = uvw * freq[j, 0]/const_c # convert from m -> lambda
	#scale = 1./(60. * 60. * 5)
	#uvw_c = uvw_c * scale
	# loop over visibility:
	for i in range(0,uvw.shape[1]):
	    if (sigma[0,i]!=-1.0)and(sigma[1,i]!=-1.0)and(sigma[2,i]!=-1.0)and(sigma[3,i]!=-1.0)and(flagXX[i]==False)and(flagXY[i]==False)and(flagYX[i]==False)and(flagYY[i]==False):
	        visfileI.write(str(uvw_c[0,i])+' '+str(uvw_c[1,i])+' '\
	                    +str(dataI[i].real)+' '+str(dataI[i].imag)\
	                    +' '+str(sigmaI[i])+'\n')
	        visfileQ.write(str(uvw_c[0,i])+' '+str(uvw_c[1,i])+' '\
	                    +str(dataQ[i].real)+' '+str(dataQ[i].imag)\
	                    +' '+str(sigmaQ[i])+'\n')
	        visfileU.write(str(uvw_c[0,i])+' '+str(uvw_c[1,i])+' '\
	                    +str(dataU[i].real)+' '+str(dataU[i].imag)\
	                    +' '+str(sigmaU[i])+'\n')
	        visfileV.write(str(uvw_c[0,i])+' '+str(uvw_c[1,i])+' '\
	                    +str(dataV[i].real)+' '+str(dataV[i].imag)\
	                    +' '+str(sigmaV[i])+'\n')
	


visfileI.close()
visfileQ.close()
visfileU.close()
visfileV.close()


tb.close



import os 
import multiprocessing
import time
import numpy as np 
import pyfits
import glob
import subprocess
import matplotlib.pyplot as plt


"""
Script that runs tests of purify on real data.
"""

nice_value = 0

home = os.path.expanduser("~")
stokes_v_calculation = False 
outpath = home + "/purify_real_data/"
if stokes_v_calculation:
	outpath = home + "/purify_real_data_v/"

data = home + "/Dropbox/Visibility_Files/"

atca = "atca/"
vla = "vla/"
names = [atca + "0332-391", atca + "0114-476", vla + "CygA-X", vla + "3C129BC"]
imsize = ["1024", "1024", "1024", "1024"]
cellsizes = ["2", "2", "0.5","0.4"]

residual_convergences = ["3700", "-1", "20000", "-1"]

def make_path(name, outpath, epsilon_factor):
	return outpath + name + "/" + str(epsilon_factor) + "/"

def make_directory(path):
	if not os.path.exists(path):
		os.makedirs(path)

def run_test((name, inpath, cellsize, size, epsilon_factor, residual_convergence)):
	path = make_path(name, outpath, epsilon_factor)
	noise = " "
	make_directory(path)
	if stokes_v_calculation == True:
		noise = " --noise " + inpath + ".ms "
        cmd = ["screen -S "+ name.split("/")[-1] + "_" + str(epsilon_factor) + 
		" -d -m ~/dev/purify/build/purify" +
		" --name " + path + name.split("/")[-1] +
		" --measurement_set " + inpath + ".ms --n_mean " + str(epsilon_factor) + 
		noise +
		" --cellsize " + cellsize +
		" --size " + str(size) +
		" --diagnostic --power_iterations 200" +
                " --residual_convergence " + residual_convergence +
                " --relative_variation 0.005" +
                " --adapt_iter 100" +
                " --niters 500" +
                " --relative_gamma_adapt 1e-3"
        ]
	proc = subprocess.Popen(cmd, shell=True)
        print cmd
	print "Waiting for " + path
	while not os.path.exists(path + name.split("/")[-1] + "_solution_whiten_final.fits"):
		time.sleep(10)
	return

params = []
for i in range(len(names)):
	i = 2
	n = names[i]
	print "Looking for " + data + n
	if os.path.exists(data + n + ".ms"):
		print n + " found!"
		image_size = imsize[i] 
		pix_size = cellsizes[i]
		params.append((n, data + n, pix_size, image_size, 7, residual_convergences[i]))
	else:
		print n + " not found!"
	break
n_processors = multiprocessing.cpu_count() + 1
p = multiprocessing.Pool(min(n_processors, 1))
p.map(run_test, params)



import os 
import multiprocessing
import time
import numpy as np 
import pyfits
import glob
import subprocess
import matplotlib.pyplot as plt

image = "30dor_256"
n_tests = 10
input_SNR = 30

def kernel_settings(kernel):
        J = 4
        oversample = 2
        if kernel == "pswf":
                J = 6
        if kernel == "kb_interp":
                oversample = 1.375
                J = 5
        if kernel == "box":
                J = 1
        if kernel == "kb_min":
                oversample = 1.375
                J = 5
	if kernel == "kb_min4":
		oversample = 1.375
		J = 4
		kernel = "kb_min"
	if kernel == "kb_interp4":
		oversample = 1.375
		J = 4
		kernel = "kb_interp"
	
	return J, oversample, kernel
def run_test_padmm_reweighted((i, kernel, M_N_ratio, start_time, input_SNR, image)):
	time.sleep(start_time)

	J, oversample, kernel = kernel_settings(kernel)

  	os.system("screen -S "+ kernel + "_" + str(i) + " -d -m " + 
	"../build/cpp/example/padmm_reweighted_simulation " + kernel + " " 
      + str(oversample) + " " +str(J) + " " +str(M_N_ratio) + " " + str(i) + " "+str(input_SNR)+ " " + str(image))
  	results_file = "../build/outputs/"+image+"_results_" + kernel + "_" + str(i) + ".txt"
	while not os.path.exists(results_file):
		time.sleep(1)
  	results = np.loadtxt(results_file, dtype = str)
  	SNR = results[0]
  	total_time = results[1]
	os.system("rm " + results_file)
  	return [float(SNR), float(total_time)]
def run_test_padmm((i, kernel, M_N_ratio, start_time, input_SNR, image)):
	time.sleep(start_time)

	J, oversample, kernel = kernel_settings(kernel)

  	os.system("screen -S "+ kernel + "_" + str(i) + " -d -m " +
	"../build/cpp/example/padmm_simulation padmm " + kernel + " " 
      + str(oversample) + " " +str(J) + " " +str(M_N_ratio) + " " + str(i) + " "+str(input_SNR)+ " " + str(image))
	results_file = "../build/outputs/"+image+"_results_" + kernel + "_" + str(i) + ".txt"
	while not os.path.exists(results_file):
		time.sleep(1)
  	results = np.loadtxt(results_file, dtype = str)
  	SNR = results[0]
  	total_time = results[1]
	converged = results[2]
	niters = results[3]
	os.system("rm " + results_file)
  	return [float(SNR), float(total_time), int(converged), float(niters)]

def run_test_ms_clean((i, kernel, M_N_ratio, start_time, input_SNR, image)):
	time.sleep(start_time)

	J, oversample, kernel = kernel_settings(kernel)

  	os.system("screen -S "+ kernel + "_" + str(i) + " -d -m " +
	"../build/cpp/example/padmm_simulation ms_clean " + kernel + " " 
      + str(oversample) + " " +str(J) + " " +str(M_N_ratio) + " " + str(i) + " "+str(input_SNR)+ " " + str(image))
	results_file = "../build/outputs/"+image+"_results_" + kernel + "_" + str(i) + ".txt"
	while not os.path.exists(results_file):
		time.sleep(1)
  	results = np.loadtxt(results_file, dtype = str)
  	SNR = results[0]
  	total_time = results[1]
	converged = results[2]
	niters = results[3]
	os.system("rm " + results_file)
  	return [float(SNR), float(total_time), int(converged), float(niters)]

def run_test_clean((i, kernel, M_N_ratio, start_time, input_SNR, image)):
	time.sleep(start_time)

	J, oversample, kernel = kernel_settings(kernel)

  	os.system("screen -S "+ kernel + "_" + str(i) + " -d -m " +
	"../build/cpp/example/padmm_simulation clean " + kernel + " " 
      + str(oversample) + " " +str(J) + " " +str(M_N_ratio) + " " + str(i) + " "+str(input_SNR)+ " " + str(image))
	results_file = "../build/outputs/"+image+"_results_" + kernel + "_" + str(i) + ".txt"
	while not os.path.exists(results_file):
		time.sleep(1)
  	results = np.loadtxt(results_file, dtype = str)
  	SNR = results[0]
  	total_time = results[1]
	converged = results[2]
	niters = results[3]
	os.system("rm " + results_file)
  	return [float(SNR), float(total_time), int(converged), float(niters)]
def collect_data(args, results, M_N_ratios, kernel):

	meantempSNR = []
	errortempSNR = []
	meantempTime = []
	errortempTime = []
	meantempIters = []
	errortempIters = []
	totaltempConverges = []
	for m in M_N_ratios:
		tempSNR = []
		tempTime = []
		tempIters = []
		tempConverges = []
		for i in range(len(args)):
			if m == args[i][2]:
				if args[i][1] == kernel:
					tempSNR.append(results[i][0])
					tempTime.append(results[i][1])
					tempConverges.append(results[i][2])
					tempIters.append(results[i][3])
		tempSNR = np.array(tempSNR)
		tempTime = np.array(tempTime)
		tempConverges = np.array(tempConverges)
		tempIters = np.array(tempIters)
		
		meantempSNR.append(tempSNR.mean())
		errortempSNR.append(tempSNR.std())
		meantempTime.append(tempTime.mean())
		errortempTime.append(tempTime.std())
		meantempIters.append(tempIters.mean())
		errortempIters.append(tempIters.std())

		totaltempConverges.append(tempConverges.sum())

	return meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges

def create_plots(args, results, M_N_ratios, name, kernels, colours,legend = []):
	for k in range(len(kernels)):
		meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges = collect_data(args, results, M_N_ratios, kernels[k])
		plt.errorbar(M_N_ratios, meantempSNR, errortempSNR, fmt='', c = colours[k])
	if len(legend) > 0:
		plt.legend(legend)
	plt.xlabel("M/N")
	plt.ylabel("SNR, db")
	plt.xlim(0, 2.2)
	plt.ylim(5, 40)
	plt.savefig(name + "_SNR_plot.pdf")
	plt.clf()

	for k in range(len(kernels)):
		meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges = collect_data(args, results, M_N_ratios, kernels[k])
		plt.errorbar(M_N_ratios, meantempTime, errortempTime, fmt='', c = colours[k])
	plt.xlabel("M/N")
	plt.ylabel("Time (seconds)")
	plt.xlim(0, 2.2)
	plt.savefig(name + "_Time_plot.pdf")
	plt.clf()


	for k in range(len(kernels)):
		meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges = collect_data(args, results, M_N_ratios, kernels[k])
		plt.errorbar(M_N_ratios, meantempIters, errortempIters, fmt='', c = colours[k])
	plt.xlabel("M/N")
	plt.ylabel("Iterations")
	plt.xlim(0, 2.2)
	plt.ylim(0, 110)
	plt.savefig(name + "_Iterations_plot.pdf")
	plt.clf()


	for k in range(len(kernels)):
		meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges = collect_data(args, results, M_N_ratios, kernels[k])
		plt.scatter(M_N_ratios, totaltempConverges, c = colours[k])
	plt.xlabel("M/N")
	plt.ylabel("Number of converging tests")
	plt.ylim(0, 11)
	plt.xlim(0, 2.2)
	plt.savefig(name + "_Converges_plot.pdf")
	plt.clf()

	for k in range(len(kernels)):
		meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges = collect_data(args, results, M_N_ratios, kernels[k])
		x = np.column_stack((meantempSNR, errortempSNR, meantempTime, errortempTime, meantempIters, errortempIters, totaltempConverges))
		np.savetxt(name + '_' + kernels[k], x)
		
if __name__ == '__main__':
	M_N_ratios = np.arange(1, 11) * 0.2
	args = []

	test_num = 0
	#kernels = ["kb", "kb_interp", "pswf", "gauss", "box", "gauss_alt", "kb_min", "kb_min4"]#, "kb_interp4"]
	kernels = ["kb", "pswf", "gauss", "gauss_alt", "box"]
	total_tests = n_tests * len(kernels) * len(M_N_ratios)
	for i in range(1, n_tests + 1):
		for k in kernels:
			for m in M_N_ratios:
				test_num = test_num + 1
				args.append((test_num, k, m, test_num * 1./ total_tests * 30., input_SNR, image))
				print test_num
	n_processors = multiprocessing.cpu_count() + 1
	p = multiprocessing.Pool(min(n_processors, 1)) # Limiting the number of processes used to 40, otherwise it will cause problems with the user limit
	
	#legend = ["Kaiser Bessel (KB)", "KB (Linear-interp, Min-oversample)", "PSWF", "Gaussian (Optimal)", "Box", "Gaussian (non-Optimal)", "KB (Min-oversample)", "KB4 (Min-oversample)"]#, "KB4 (Linear-interp, Min-oversample)"]
	#colours = ['blue', 'red', 'black', 'green', 'magenta', 'cyan', 'yellow', "#800000"]#, "#808000"]
	legend = ["Kaiser Bessel (KB)", "PSWF", "Gaussian (Optimal)", "Gaussian (non-Optimal)", "Box"]
	colours = ['blue', 'black', 'green', 'cyan', 'magenta']
	results = p.map(run_test_padmm, args)
	create_plots(args, results, M_N_ratios, image + "_padmm_" + str(input_SNR) + "_", kernels, colours, legend)
	print "PADMM Done!"
	results = p.map(run_test_ms_clean, args)
	create_plots(args, results, M_N_ratios, image + "_ms_clean" + str(input_SNR) + "_", kernels, colours)
	print "MS CLEAN Done!"
	results = p.map(run_test_clean, args)
	create_plots(args, results, M_N_ratios, image + "_clean" + str(input_SNR) + "_", kernels, colours)
	print "CLEAN Done!"
	#results = p.map(run_test_padmm_reweighted, args)
	#create_plots(args, results, M_N_ratios, image + "_padmm_reweighted" + str(input_SNR) + "_", kernels, colours, legend)
	#print "PADMM REWEIGHTED Done!"

